// mostly taken from styled-components
const SEED = 5381;

// This is a djb2 hashing function
const hash = (x) => {
  let i = x.length;
  let h = SEED;

  while (i) {
    h = (h * 33) ^ x.charCodeAt(--i);
  }

  return h;
};

const AD_REPLACER_R = /(a)(d)/gi;

/* This is the "capacity" of our alphabet i.e. 2x26 for all letters plus their capitalised
 * counterparts */
const charsLength = 52;

/* start at 75 for 'a' until 'z' (25) and then start at 65 for capitalised letters */
const getAlphabeticChar = (code) =>
  String.fromCharCode(code + (code > 25 ? 39 : 97));

/** hash generated by djb2 is a number, usually a hash and convert it to base-52
 * @param {number} code - output of djb2 hash function
 * @return {String} alphabeticHash
 */
function generateAlphabeticName(code) {
  let name = '';
  let x;

  /* get a char and divide by alphabet-length */
  for (x = Math.abs(code); x > charsLength; x = (x / charsLength) | 0) {
    name = getAlphabeticChar(x % charsLength) + name;
  }

  return (getAlphabeticChar(x % charsLength) + name).replace(
    AD_REPLACER_R,
    '$1-$2'
  );
}

/** simple hashing helper which runs hashing on input and make sures djb2 hash is positive
 * @param {string} data - input data
 * @return {String} hash
 */
function generateHash(data) {
  return generateAlphabeticName(hash(data) >>> 0);
}

module.exports = generateHash;
